//
//  ContentView.swift
//  ExampleTCA
//
//  Created by Jean-Nicolas Jolivet on 2023-03-31.
//

import SwiftUI
import ComposableArchitecture
struct DetailsFeature: Reducer {
    struct State: Equatable {
        var item: Item
        var counter = 0
    }
    enum Action: Equatable {
        case tappedIncrease
    }
    
    func reduce(into state: inout State, action: Action) -> Effect<Action> {
        switch action {
        case .tappedIncrease:
            state.counter += 1
            return .none
        }
    }
}


struct DetailsView: View {
    let store: StoreOf<DetailsFeature>
    var body: some View {
        WithViewStore(
            self.store,
            observe: { $0 }
        ) { viewStore in
            VStack {
                Text("Details view for: \(viewStore.item.name)")
                Text("Count: \(viewStore.state.counter)")
                Button("Increase") {
                    viewStore.send(.tappedIncrease)
                }
            }.onAppear {
                viewStore.send(.tappedIncrease)
            }
        }
    }
}

struct DetailsView_Previews: PreviewProvider {
    static var previews: some View {
        DetailsView(store: StoreOf<DetailsFeature>(
            initialState: .init(item: Item.mock()),
            reducer: DetailsFeature() )
        )
    }
}
