//
//  ExampleTCAApp.swift
//  ExampleTCA
//
//  Created by Jean-Nicolas Jolivet on 2023-03-31.
//

import ComposableArchitecture
import SwiftUI

@main
struct ExampleTCAApp: App {
    var body: some Scene {
        WindowGroup {
            RootView(store: .init(
                initialState: .init(items: .init(uniqueElements: [.mock(), .mock(), .mock()])),
                reducer: RootFeature())
            )
        }
    }
}

enum Tab {
    case first, second
}

struct RootFeature: Reducer {
    struct State: Equatable {
        var items: IdentifiedArrayOf<Item>
        @PresentationState var details: DetailsFeature.State?
    }

    enum Action: Equatable {
        case tappedItem(Item)
        case details(PresentationAction<DetailsFeature.Action>)
    }

    var body: some ReducerOf<Self> {
        Reduce<State, Action> { state, action in
            switch action {
            case .details:
                return .none
            case let .tappedItem(item):
                state.details = DetailsFeature.State(item: item)
                return .none
            }
        }.ifLet(\.$details, action: /Action.details) {
            DetailsFeature()
        }._printChanges()
        
    }
}

struct RootView: View {
    let store: StoreOf<RootFeature>
    var body: some View {
        NavigationStack {
            WithViewStore(self.store, observe: { $0 }) { viewStore in
                VStack {
                    ForEach(viewStore.items) { item in
                        Text(item.name)
                            .onTapGesture {
                                viewStore.send(.tappedItem(item))
                            }.navigationDestination(
                                store: self.store.scope(state: \.$details, action: RootFeature.Action.details)) { store in
                                    DetailsView(store: store)
                                }
                    }
                }
            }
        }
    }
}

struct Item: Equatable, Identifiable {
    let id = UUID()
    let name: String
    static var count = 0

    public static func mock() -> Item {
        defer { Self.count += 1 }
        return Item(name: "Item \(Self.count)")
    }
}

struct RootView_Previews: PreviewProvider {
    static var previews: some View {
        RootView(store: .init(
            initialState: .init(items: .init(uniqueElements: [.mock(), .mock(), .mock()])),
            reducer: RootFeature()))
    }
}
